"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorldManager = void 0;
class Column {
}
class Table {
}
class WorldManager {
    constructor(appConfig, tokenGetter, jsonApi, actionManager, axios) {
        this.appConfig = appConfig;
        this.jsonApi = jsonApi;
        this.axios = axios;
        this.actionManager = actionManager;
        this.tokenGetter = tokenGetter;
        this.columnKeysCache = {};
        this.stateMachineEnabled = {};
        this.stateMachines = {};
        this.systemActions = {};
        this.worlds = {};
        this.columnTypes = {};
    }
    modelLoader(typeName, force) {
        const that = this;
        return new Promise(function (resolve, reject) {
            return that.getColumnKeys(typeName, force).then(resolve).catch(reject);
        });
    }
    ;
    init() {
        const that = this;
        return new Promise(function (resolve, reject) {
            that.columnTypes = [];
            that.axios(that.appConfig.getEndpoint() + "/meta?query=column_types", {
                headers: {
                    "Authorization": "Bearer " + that.tokenGetter.getToken()
                }
            }).then(function (r) {
                if (r.status === 200) {
                    that.columnTypes = r.data;
                    resolve(r.data);
                }
                else {
                    reject(r.data);
                }
            }, reject);
            that.jsonApi.define("image.png|jpg|jpeg|gif|tiff", {
                "__type": "value",
                "contents": "value",
                "name": "value",
                "reference_id": "value",
                "src": "value",
                "type": "value"
            });
        });
    }
    getStateMachinesForType(typeName) {
        const that = this;
        return new Promise(function (resolve, reject) {
            resolve(that.stateMachines[typeName]);
        });
    }
    ;
    startObjectTrack(objType, objRefId, stateMachineRefId) {
        const that = this;
        return new Promise(function (resolve, reject) {
            that.axios({
                url: that.appConfig.getEndpoint() + "/track/start/" + stateMachineRefId,
                method: "POST",
                data: {
                    typeName: objType,
                    referenceId: objRefId
                },
                headers: {
                    "Authorization": "Bearer " + that.tokenGetter.getToken()
                }
            }).then(function (response) {
                resolve(response.data);
            }, function (response) {
                reject(response.data);
            });
        });
    }
    ;
    trackObjectEvent(typeName, stateMachineRefId, eventName) {
        const that = this;
        return new Promise(function (resolve, reject) {
            that.axios({
                url: that.appConfig.getEndpoint() + "/track/event/" + typeName + "/" + stateMachineRefId + "/" + eventName,
                method: "POST",
                headers: {
                    "Authorization": "Bearer " + that.tokenGetter.getToken()
                }
            }).then(function (response) {
                resolve(response.data);
            }, function (response) {
                reject(response.data);
            });
        });
    }
    ;
    getColumnKeys(typeName, force) {
        const that = this;
        return new Promise(function (resolve, reject) {
            if (that.columnKeysCache[typeName] && !force) {
                resolve(that.columnKeysCache[typeName]);
                return;
            }
            that.axios(that.appConfig.getEndpoint() + '/jsmodel/' + typeName + ".js", {
                headers: {
                    "Authorization": "Bearer " + that.tokenGetter.getToken()
                },
            }).then(function (response) {
                if (response.status === 200) {
                    const data = response.data;
                    if (data.Actions.length > 0) {
                        console.log("Register actions", typeName, data.Actions);
                        that.actionManager.addAllActions(data.Actions);
                    }
                    that.stateMachines[typeName] = data.StateMachines;
                    that.stateMachineEnabled[typeName] = data.IsStateMachineEnabled;
                    that.columnKeysCache[typeName] = data;
                    resolve(data);
                }
                else {
                    reject({ error: response.data });
                }
            }, function (e) {
                reject({ error: e });
            });
        });
    }
    ;
    getColumnFieldTypes() {
        const that = this;
        return that.columnTypes;
    }
    isStateMachineEnabled(typeName) {
        const that = this;
        return that.stateMachineEnabled[typeName] === true;
    }
    ;
    GetJsonApiModel(columnModel) {
        const that = this;
        const model = {};
        if (!columnModel) {
            return model;
        }
        const keys = Object.keys(columnModel);
        for (let i = 0; i < keys.length; i++) {
            const key = keys[i];
            const data = columnModel[key];
            if (data["jsonApi"]) {
                model[key] = data;
            }
            else {
                model[key] = data.ColumnType;
            }
        }
        return model;
    }
    ;
    getWorlds() {
        const that = this;
        return that.worlds;
    }
    ;
    getWorldByName(name) {
        const that = this;
        return that.worlds[name];
    }
    ;
    getSystemActions() {
        const that = this;
        return that.systemActions;
    }
    loadModel(modelName, force) {
        if (!(modelName instanceof Array)) {
            modelName = [modelName];
        }
        const that = this;
        return Promise.all(modelName.map(function (mName) {
            return new Promise(function (resolve, reject) {
                that.modelLoader(mName, force).then(function (columnKeys) {
                    that.jsonApi.define(mName, that.GetJsonApiModel(columnKeys.ColumnModel));
                    resolve(that.GetJsonApiModel(columnKeys.ColumnModel));
                }).catch(reject);
            });
        }));
    }
    loadStreams(force) {
        const that = this;
        return new Promise(function (resolve, reject) {
            that.modelLoader("stream", force).then(function (streamKeys) {
                that.jsonApi.define("stream", that.GetJsonApiModel(streamKeys.ColumnModel));
                that.jsonApi.findAll('stream', {
                    page: { number: 1, size: 500 },
                }).then(function (res) {
                    res = res.data;
                    that.streams = res;
                    console.log("Get all streams result", res);
                    Promise.all(that.streams.map(function (stream) {
                        return new Promise(function (resolve, reject) {
                            that.modelLoader(stream.stream_name, force).then(function (model) {
                                console.log("Loaded stream model", stream.stream_name, model);
                                that.jsonApi.define(stream.stream_name, that.GetJsonApiModel(model.ColumnModel));
                                resolve(that.GetJsonApiModel(model.ColumnModel));
                            }).catch(reject);
                        });
                    })).then(function () {
                        resolve(res);
                    }).catch(reject);
                });
            });
        });
    }
    refreshWorld(worldName, force) {
        const that = this;
        return new Promise(function (resolve, reject) {
            if (worldName.indexOf("_has_") > -1) {
                resolve(null);
                return;
            }
            var promises = [];
            if (!that.worlds[worldName]) {
                promises.push(that.jsonApi.findAll('world', {
                    page: { number: 1, size: 1 },
                    query: [{ "column": "table_name", "operator": "is", "value": worldName }],
                }).then(function (res) {
                    res = res.data;
                    let total = res.length;
                    if (total == 0) {
                        reject({ error: "no results from server" });
                    }
                    that.worlds[res[0].table_name] = res[0];
                    resolve(res[0]);
                }));
            }
            promises.push(that.modelLoader(worldName, force).then(function (model) {
                let jsonApiModel = that.GetJsonApiModel(model.ColumnModel);
                that.jsonApi.define(worldName, jsonApiModel);
                resolve(jsonApiModel);
            }).catch(reject));
            return Promise.all(promises);
        });
    }
    refreshWorlds(force) {
        const that = this;
        return new Promise(function (resolve, reject) {
            that.jsonApi.findAll('world', {
                page: { number: 1, size: 500 }
            }).then(function (res) {
                res = res.data;
                let total = res.length;
                if (total == 0) {
                    reject({ error: "no tables found" });
                }
                Promise.all(res.map(function (world) {
                    that.worlds[world.table_name] = world;
                    return that.refreshWorld(world.table_name, force);
                })).then(function () {
                    resolve(res);
                }).catch(reject);
            });
        });
    }
    loadModels(force) {
        const that = this;
        return new Promise(function (resolve, reject) {
            return __awaiter(this, void 0, void 0, function* () {
                if (force) {
                    that.worlds = {};
                }
                const userAccountDef = yield that.modelLoader("user_account", force);
                that.jsonApi.define("user_account", that.GetJsonApiModel(userAccountDef.ColumnModel));
                const userGroupDef = yield that.modelLoader("usergroup", force);
                that.jsonApi.define("usergroup", that.GetJsonApiModel(userGroupDef.ColumnModel));
                const worldDef = yield that.modelLoader("world", force);
                that.systemActions = worldDef.Actions;
                that.jsonApi.define("world", that.GetJsonApiModel(worldDef.ColumnModel));
                that.refreshWorlds(force).then(function () {
                    resolve(that.worlds);
                }).catch(reject);
            });
        });
    }
}
exports.WorldManager = WorldManager;
exports.default = WorldManager;
