"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DaptinClient = void 0;
const axios_1 = require("axios");
const actionmanager_1 = require("./clients/actionmanager");
const appconfig_1 = require("./clients/appconfig");
const statsmanager_1 = require("./clients/statsmanager");
const worldmanager_1 = require("./clients/worldmanager");
const configmanager_1 = require("./clients/configmanager");
const aggregate_client_1 = require("./clients/aggregate_client");
const JsonApi = require("devour-client");
class LocalStorageTokenGetter {
    getToken() {
        return localStorage.getItem("token");
    }
}
class DaptinClient {
    constructor(endpoint, debug, tokenGetter, axiosConfig) {
        const that = this;
        debug = debug || false;
        axiosConfig = axiosConfig || {};
        let axiosInstance = axios_1.default.create(axiosConfig);
        that.appConfig = new appconfig_1.AppConfig(endpoint);
        that.jsonApi = new JsonApi(Object.assign({ apiUrl: that.appConfig.getEndpoint() + '/api', pluralize: false, logger: debug }, axiosConfig));
        that.jsonApi.axios = axiosInstance;
        that.tokenGetter = tokenGetter || new LocalStorageTokenGetter();
        that.actionManager = new actionmanager_1.ActionManager(that.appConfig, that.tokenGetter, axiosInstance);
        that.worldManager = new worldmanager_1.WorldManager(that.appConfig, that.tokenGetter, that.jsonApi, that.actionManager, axiosInstance);
        that.statsManager = new statsmanager_1.StatsManager(that.appConfig, that.tokenGetter, axiosInstance);
        that.configManager = new configmanager_1.ConfigManager(that.appConfig, that.tokenGetter, axiosInstance);
        that.aggregateClient = new aggregate_client_1.default(that.appConfig, that.tokenGetter, axiosInstance);
        that.jsonApi.insertMiddlewareBefore("HEADER", {
            name: "Auth Header middleware",
            req: function (req) {
                let token = that.tokenGetter.getToken();
                if (token) {
                    that.jsonApi.headers['Authorization'] = 'Bearer ' + token;
                }
                return req;
            }
        });
    }
}
exports.DaptinClient = DaptinClient;
